@@ -26,6 +26,7 @@
 #include "pysidemacros.h"
 #include <Python.h>
 #include <QMetaObject>
+#include <QMetaMethod>
 
 namespace PySide
 {
@@ -34,20 +35,31 @@ class DynamicQMetaObject : public QMetaObject
 {
 public:
     DynamicQMetaObject(const char* className, const QMetaObject* metaObject);
+    DynamicQMetaObject(PyTypeObject* type, const QMetaObject* metaobject);
     ~DynamicQMetaObject();
 
-    void addSignal(const char* signal, const char* type = 0);
-    void addSlot(const char* slot, const char* type = 0);
-    void addProperty(const char* property, PyObject* data);
+
+    int addMethod(QMetaMethod::MethodType mtype, const char* signature, const char* type);
+    void removeMethod(QMetaMethod::MethodType mtype, uint index);
+    int addSignal(const char* signal, const char* type = 0);
+    int addSlot(const char* slot, const char* type = 0);
+    int addProperty(const char* property, PyObject* data);
+    void addInfo(const char* key, const char* value);
+    void addInfo(QMap<QByteArray, QByteArray> info);
 
     void removeSignal(uint idex);
     void removeSlot(uint index);
     void removeProperty(uint index);
 
+    const QMetaObject* update() const;
+
 private:
     class DynamicQMetaObjectPrivate;
     DynamicQMetaObjectPrivate* m_d;
+
+    void parsePythonType(PyTypeObject* type);
 };
 
+
 }
 #endif
