@@ -22,10 +22,12 @@
 
 #include "globalreceiver.h"
 #include "dynamicqmetaobject_p.h"
+#include "pysideweakref.h"
 
 #include <QMetaMethod>
 #include <QDebug>
 #include <QEvent>
+#include <QLinkedList>
 #include <autodecref.h>
 #include <gilstate.h>
 
@@ -39,41 +41,75 @@ namespace PySide
 class DynamicSlotData
 {
     public:
-        DynamicSlotData(int id, PyObject* callback);
+        DynamicSlotData(int id, PyObject* callback, GlobalReceiver* parent);
         void addRef(const QObject* o);
         void decRef(const QObject* o);
         void clear();
-        bool hasRefTo(const QObject* o) const;
+        int hasRefTo(const QObject* o) const;
         int refCount() const;
         int id() const;
-        PyObject* callback() const;
+        PyObject* call(PyObject* args);
         ~DynamicSlotData();
+        static void onCallbackDestroyed(void* data);
 
     private:
         int m_id;
+        bool m_isMethod;
         PyObject* m_callback;
-        QSet<const QObject*> m_refs;
+        PyObject* m_pythonSelf;
+        PyObject* m_pyClass;
+        PyObject* m_weakRef;
+        GlobalReceiver* m_parent;
+        QLinkedList<const QObject*> m_refs;
 };
 
 }
 
 using namespace PySide;
 
-DynamicSlotData::DynamicSlotData(int id, PyObject* callback)
-    : m_id(id)
+DynamicSlotData::DynamicSlotData(int id, PyObject* callback, GlobalReceiver* parent)
+    : m_id(id), m_pythonSelf(0), m_pyClass(0), m_weakRef(0), m_parent(parent)
 {
-    m_callback = callback;
-    Py_INCREF(callback);
+    m_isMethod = PyMethod_Check(callback);
+    if (m_isMethod) {
+        //Can not store calback pointe because this will be destroyed at the end of the scope
+        //To avoid increment intance reference keep the callback information
+        m_callback = PyMethod_GET_FUNCTION(callback);
+        m_pyClass = PyMethod_GET_CLASS(callback);
+        m_pythonSelf = PyMethod_GET_SELF(callback);
+
+        //monitor class from method lifetime
+        m_weakRef = WeakRef::create(m_pythonSelf, DynamicSlotData::onCallbackDestroyed, this);
+    } else {
+        m_callback = callback;
+        Py_INCREF(m_callback);
+    }
+}
+
+PyObject* DynamicSlotData::call(PyObject* args)
+{
+    PyObject* callback = m_callback;
+
+    //create a callback based on method data
+    if (m_isMethod)
+        callback = PyMethod_New(callback, m_pythonSelf, m_pyClass);
+
+    PyObject* result =  PyObject_CallObject(callback, args);
+
+    if (m_isMethod)
+        Py_DECREF(callback);
+
+    return result;
 }
 
 void DynamicSlotData::addRef(const QObject *o)
 {
-    m_refs.insert(o);
+    m_refs.append(o);
 }
 
 void DynamicSlotData::decRef(const QObject *o)
 {
-    m_refs.remove(o);
+    m_refs.removeOne(o);
 }
 
 int DynamicSlotData::refCount() const
@@ -81,31 +117,44 @@ int DynamicSlotData::refCount() const
     return m_refs.size();
 }
 
-
-PyObject* DynamicSlotData::callback() const
-{
-    return m_callback;
-}
-
 int DynamicSlotData::id() const
 {
     return m_id;
 }
 
-bool DynamicSlotData::hasRefTo(const QObject *o) const
+int DynamicSlotData::hasRefTo(const QObject *o) const
 {
-    return m_refs.contains(o);
+    return m_refs.count(o);
 }
 
 void DynamicSlotData::clear()
 {
+    Shiboken::GilState gil;
+    Py_XDECREF(m_weakRef);
+    m_weakRef = 0;
     m_refs.clear();
 }
 
 DynamicSlotData::~DynamicSlotData()
 {
     Shiboken::GilState gil;
-    Py_DECREF(m_callback);
+    clear();
+    if (!m_isMethod)
+       Py_DECREF(m_callback);
+}
+
+void DynamicSlotData::onCallbackDestroyed(void *data)
+{
+    Shiboken::GilState gil;
+    DynamicSlotData* self = reinterpret_cast<DynamicSlotData*>(data);
+
+    //Disconnect all sources
+    QMetaMethod m = self->m_parent->metaObject()->method(self->m_id);
+    QByteArray methodName = QByteArray::number(m.methodType()).append(m.signature());
+    QLinkedList<const QObject*> sources = self->m_refs;
+    foreach(const QObject* src, sources)
+        const_cast<QObject*>(src)->disconnect(self->m_parent, methodName);
+    self->m_weakRef = 0;
 }
 
 GlobalReceiver::GlobalReceiver()
@@ -113,6 +162,8 @@ GlobalReceiver::GlobalReceiver()
 {
     //slot used to be notifyed of object destrouction
     m_metaObject.addSlot(RECEIVER_DESTROYED_SLOT_NAME);
+    m_metaObject.update();
+    setObjectName("GLOBAL RECEIVER");
 }
 
 GlobalReceiver::~GlobalReceiver()
@@ -149,15 +200,14 @@ void GlobalReceiver::disconnectNotify(QObject* source, int slotId)
 
 const QMetaObject* GlobalReceiver::metaObject() const
 {
-    return &m_metaObject;
+    return m_metaObject.update();
 }
 
-void GlobalReceiver::addSlot(const char* slot, PyObject* callback)
+int GlobalReceiver::addSlot(const char* slot, PyObject* callback)
 {
-    m_metaObject.addSlot(slot);
-    int slotId = m_metaObject.indexOfSlot(slot);
+    int slotId = m_metaObject.addSlot(slot);
     if (!m_slotReceivers.contains(slotId))
-        m_slotReceivers[slotId] = new DynamicSlotData(slotId, callback);
+        m_slotReceivers[slotId] = new DynamicSlotData(slotId, callback, this);
 
     bool isShortCircuit = true;
     for (int i = 0; slot[i]; ++i) {
@@ -170,8 +220,8 @@ void GlobalReceiver::addSlot(const char* slot, PyObject* callback)
     if (isShortCircuit)
         m_shortCircuitSlots << slotId;
 
-
     Q_ASSERT(slotId >= QObject::staticMetaObject.methodCount());
+    return slotId;
 }
 
 void GlobalReceiver::removeSlot(int slotId)
@@ -199,7 +249,7 @@ int GlobalReceiver::qt_metacall(QMetaObject::Call call, int id, void** args)
 {
     Q_ASSERT(call == QMetaObject::InvokeMetaMethod);
     Q_ASSERT(id >= QObject::staticMetaObject.methodCount());
-    QMetaMethod slot = m_metaObject.method(id);
+    QMetaMethod slot = metaObject()->method(id);
     Q_ASSERT(slot.methodType() == QMetaMethod::Slot);
 
     if (strcmp(slot.signature(), RECEIVER_DESTROYED_SLOT_NAME) == 0) {
@@ -209,9 +259,11 @@ int GlobalReceiver::qt_metacall(QMetaObject::Call call, int id, void** args)
         QHash<int, DynamicSlotData*> copy = m_slotReceivers;
         QHash<int, DynamicSlotData*>::iterator i = copy.begin();
         while(i != copy.end()) {
-            if (i.value()->hasRefTo(arg)) {
+            //Remove all refs
+            int refs = i.value()->hasRefTo(arg);
+            while(refs) {
                 disconnectNotify(arg, i.key());
-                break;
+                refs--;
             }
             i++;
         }
@@ -225,21 +277,17 @@ int GlobalReceiver::qt_metacall(QMetaObject::Call call, int id, void** args)
     }
 
     Shiboken::GilState gil;
-    int numArgs;
     PyObject* retval = 0;
-    PyObject* callback = data->callback();
     if (m_shortCircuitSlots.contains(id)) {
-        retval = PyObject_CallObject(callback, reinterpret_cast<PyObject*>(args[1]));
+        retval = data->call(reinterpret_cast<PyObject*>(args[1]));
     } else {
         QList<QByteArray> paramTypes = slot.parameterTypes();
-        numArgs = paramTypes.count();
         Shiboken::AutoDecRef preparedArgs(PyTuple_New(paramTypes.count()));
         for (int i = 0, max = paramTypes.count(); i < max; ++i) {
-            PyObject* arg = Shiboken::TypeResolver::get(paramTypes[i].constData())->toPython(args[i+1]);
+            PyObject* arg = Shiboken::TypeResolver::get(paramTypes[i].constData())->toPython(args[i+1]); // Do not increment the reference
             PyTuple_SET_ITEM(preparedArgs.object(), i, arg);
         }
-
-        retval = PyObject_CallObject(callback, preparedArgs);
+        retval = data->call(preparedArgs);
     }
 
     if (!retval)
@@ -249,3 +297,4 @@ int GlobalReceiver::qt_metacall(QMetaObject::Call call, int id, void** args)
 
     return -1;
 }
+
