@@ -22,6 +22,7 @@
 
 #include "pyside.h"
 #include "signalmanager.h"
+#include "pysideclassinfo_p.h"
 #include "pysideproperty_p.h"
 #include "pysideproperty.h"
 #include "pysidesignal.h"
@@ -30,6 +31,7 @@
 #include "pysidemetafunction_p.h"
 #include "pysidemetafunction.h"
 #include "dynamicqmetaobject.h"
+#include "destroylistener.h"
 
 #include <basewrapper.h>
 #include <conversions.h>
@@ -39,14 +41,18 @@
 #include <cctype>
 #include <QStack>
 #include <QCoreApplication>
+#include <QDebug>
 
 static QStack<PySide::CleanupFunction> cleanupFunctionList;
+static void* qobjectNextAddr;
 
 namespace PySide
 {
 
 void init(PyObject *module)
 {
+    qobjectNextAddr = 0;
+    ClassInfo::init(module);
     Signal::init(module);
     Slot::init(module);
     Property::init(module);
@@ -74,7 +80,7 @@ bool fillQtProperties(PyObject* qObj, const QMetaObject* metaObj, PyObject* kwds
                     Shiboken::AutoDecRef retval(PyObject_CallObject(propSetter, args));
                 } else {
                     PyObject* attr = PyObject_GenericGetAttr(qObj, key);
-                    if (PySide::Property::isPropertyType(attr))
+                    if (PySide::Property::checkType(attr))
                         PySide::Property::setValue(reinterpret_cast<PySideProperty*>(attr), qObj, value);
                 }
             } else {
@@ -99,10 +105,12 @@ void registerCleanupFunction(CleanupFunction func)
 
 void runCleanupFunctions()
 {
+    //PySide::DestroyListener::instance()->destroy();
     while (!cleanupFunctionList.isEmpty()) {
         CleanupFunction f = cleanupFunctionList.pop();
         f();
     }
+    PySide::DestroyListener::destroy();
 }
 
 static void destructionVisitor(SbkObject* pyObj, void* data)
@@ -113,12 +121,14 @@ static void destructionVisitor(SbkObject* pyObj, void* data)
 
     if (pyObj != pyQApp && PyObject_TypeCheck(pyObj, pyQObjectType)) {
         if (Shiboken::Object::hasOwnership(pyObj) && Shiboken::Object::isValid(pyObj, false)) {
+            Shiboken::Object::setValidCpp(pyObj, false);
+
             Py_BEGIN_ALLOW_THREADS
             Shiboken::callCppDestructor<QObject>(Shiboken::Object::cppPointer(pyObj, pyQObjectType));
             Py_END_ALLOW_THREADS
-            Shiboken::Object::setValidCpp(pyObj, false);
         }
     }
+
 };
 
 void destroyQCoreApplication()
@@ -140,11 +150,36 @@ void destroyQCoreApplication()
     delete app;
 }
 
+struct TypeUserData {
+    TypeUserData(PyTypeObject* type, const QMetaObject* metaobject) : mo(type, metaobject) {}
+    DynamicQMetaObject mo;
+    std::size_t cppObjSize;
+};
+
+std::size_t getSizeOfQObject(SbkObjectType* type)
+{
+    using namespace Shiboken::ObjectType;
+    TypeUserData* userData = reinterpret_cast<TypeUserData*>(getTypeUserData(reinterpret_cast<SbkObjectType*>(type)));
+    return userData->cppObjSize;
+}
+
+void initDynamicMetaObject(SbkObjectType* type, const QMetaObject* base, const std::size_t& cppObjSize)
+{
+    //create DynamicMetaObject based on python type
+    TypeUserData* userData = new TypeUserData(reinterpret_cast<PyTypeObject*>(type), base);
+    userData->cppObjSize = cppObjSize;
+    userData->mo.update();
+    Shiboken::ObjectType::setTypeUserData(type, userData, Shiboken::callCppDestructor<TypeUserData>);
+
+    //initialize staticQMetaObject property
+    void* metaObjectPtr = &userData->mo;
+    Shiboken::AutoDecRef pyMetaObject(Shiboken::TypeResolver::get("QMetaObject*")->toPython(&metaObjectPtr));
+    PyObject_SetAttrString(reinterpret_cast<PyObject*>(type), "staticMetaObject", pyMetaObject);
+}
+
 void initDynamicMetaObject(SbkObjectType* type, const QMetaObject* base)
 {
-    QByteArray typeName = QByteArray(type->super.ht_type.tp_name).split('.').last();
-    DynamicQMetaObject* mo = new PySide::DynamicQMetaObject(typeName, base);
-    Shiboken::ObjectType::setTypeUserData(type, mo, &Shiboken::callCppDestructor<DynamicQMetaObject>);
+    initDynamicMetaObject(type, base, 0);
 }
 
 void initQObjectSubType(SbkObjectType* type, PyObject* args, PyObject* kwds)
@@ -153,17 +188,16 @@ void initQObjectSubType(SbkObjectType* type, PyObject* args, PyObject* kwds)
     QByteArray className(PyString_AS_STRING(PyTuple_GET_ITEM(args, 0)));
 
     PyObject* bases = PyTuple_GET_ITEM(args, 1);
-    int numBases = PyTuple_GET_SIZE(args);
+    int numBases = PyTuple_GET_SIZE(bases);
     QMetaObject* baseMo = 0;
+    SbkObjectType* qobjBase = 0;
 
     for (int i = 0; i < numBases; ++i) {
         PyTypeObject* base = reinterpret_cast<PyTypeObject*>(PyTuple_GET_ITEM(bases, i));
         if (PyType_IsSubtype(base, qObjType)) {
             baseMo = reinterpret_cast<QMetaObject*>(Shiboken::ObjectType::getTypeUserData(reinterpret_cast<SbkObjectType*>(base)));
-            // If it's a class like QObject, QWidget, etc, use the original QMetaObject instead of the dynamic one
-            // IMO this if is a bug, however it keeps the current behaviour.
-            if (!Shiboken::ObjectType::isUserType(base))
-                baseMo = const_cast<QMetaObject*>(baseMo->d.superdata);
+            qobjBase = reinterpret_cast<SbkObjectType*>(base);
+            reinterpret_cast<DynamicQMetaObject*>(baseMo)->update();
             break;
         }
     }
@@ -172,71 +206,21 @@ void initQObjectSubType(SbkObjectType* type, PyObject* args, PyObject* kwds)
         return;
     }
 
-    DynamicQMetaObject* mo = new PySide::DynamicQMetaObject(className.constData(), baseMo);
-
-    Shiboken::ObjectType::setTypeUserData(type, mo, &Shiboken::callCppDestructor<DynamicQMetaObject>);
-
-    PyObject* attrs = PyTuple_GET_ITEM(args, 2);
-    PyObject* key;
-    PyObject* value;
-    Py_ssize_t pos = 0;
-
-    typedef std::pair<const char*, PyObject*> PropPair;
-    QList<PropPair> properties;
-
-    Shiboken::AutoDecRef slotAttrName(PyString_FromString(PYSIDE_SLOT_LIST_ATTR));
-
-    while (PyDict_Next(attrs, &pos, &key, &value)) {
-        if (PyType_IsSubtype(value->ob_type, &PySidePropertyType)) {
-            // Leave the properties to be register after signals because they may depend on notify signals
-            properties << PropPair(PyString_AS_STRING(key), value);
-        } else if (value->ob_type == &PySideSignalType) { // Register signals
-            PySideSignal* data = reinterpret_cast<PySideSignal*>(value);
-            const char* signalName = PyString_AS_STRING(key);
-            data->signalName = strdup(signalName);
-            QByteArray sig;
-            sig.reserve(128);
-            for (int i = 0; i < data->signaturesSize; ++i) {
-                sig = signalName;
-                sig += '(';
-                if (data->signatures[i])
-                    sig += data->signatures[i];
-                sig += ')';
-                if (baseMo->indexOfSignal(sig) == -1)
-                    mo->addSignal(sig);
-            }
-        } else if (PyFunction_Check(value)) { // Register slots
-            if (PyObject_HasAttr(value, slotAttrName)) {
-                PyObject* signatureList = PyObject_GetAttr(value, slotAttrName);
-                for(Py_ssize_t i = 0, i_max = PyList_Size(signatureList); i < i_max; ++i) {
-                    PyObject* signature = PyList_GET_ITEM(signatureList, i);
-                    QByteArray sig(PyString_AS_STRING(signature));
-                    //slot the slot type and signature
-                    QList<QByteArray> slotInfo = sig.split(' ');
-                    int index = baseMo->indexOfSlot(slotInfo[1]);
-                    if (index == -1)
-                        mo->addSlot(slotInfo[1], slotInfo[0]);
-                }
-            }
-        }
-    }
-
-    // Register properties
-    foreach (PropPair propPair, properties)
-        mo->addProperty(propPair.first, propPair.second);
+    TypeUserData* userData = reinterpret_cast<TypeUserData*>(Shiboken::ObjectType::getTypeUserData(qobjBase));
+    initDynamicMetaObject(type, baseMo, userData->cppObjSize);
 }
 
 PyObject* getMetaDataFromQObject(QObject* cppSelf, PyObject* self, PyObject* name)
 {
     PyObject* attr = PyObject_GenericGetAttr(self, name);
-    if (!Shiboken::Object::isValid(reinterpret_cast<SbkObject*>(self), false)) 
+    if (!Shiboken::Object::isValid(reinterpret_cast<SbkObject*>(self), false))
         return attr;
 
-    if (attr && Property::isPropertyType(attr)) {
+    if (attr && Property::checkType(attr)) {
         PyObject *value = Property::getValue(reinterpret_cast<PySideProperty*>(attr), self);
+        Py_DECREF(attr);
         if (!value)
             return 0;
-        Py_DECREF(attr);
         Py_INCREF(value);
         attr = value;
     }
@@ -278,5 +262,27 @@ PyObject* getMetaDataFromQObject(QObject* cppSelf, PyObject* self, PyObject* nam
     return attr;
 }
 
+bool inherits(PyTypeObject* objType, const char* class_name)
+{
+    if (strcmp(objType->tp_name, class_name) == 0)
+        return true;
+
+    PyTypeObject* base = (objType)->tp_base;
+    if (base == 0)
+        return false;
+
+    return inherits(base, class_name);
+}
+
+void* nextQObjectMemoryAddr()
+{
+    return qobjectNextAddr;
+}
+
+void setNextQObjectMemoryAddr(void* addr)
+{
+    qobjectNextAddr = addr;
+}
+
 } //namespace PySide
 
