@@ -28,6 +28,7 @@
 #include <QDebug>
 
 #define SIGNAL_CLASS_NAME "Signal"
+#define SIGNAL_INSTANCE_NAME "SignalInstance"
 #define QT_SIGNAL_SENTINEL '2'
 
 
@@ -60,15 +61,57 @@ static PyObject* signalInstanceGetItem(PyObject*, PyObject*);
 static PyObject* signalInstanceCall(PyObject* self, PyObject* args, PyObject* kw);
 static PyObject* signalCall(PyObject*, PyObject*, PyObject*);
 
+static PyObject* metaSignalCheck(PyObject*, PyObject*);
+
 static PyMappingMethods Signal_as_mapping = {
     0,
     signalGetItem,
     0
 };
 
+static PyMethodDef Signal_methods[] = {
+    {"__instancecheck__", (PyCFunction)metaSignalCheck, METH_O, NULL},
+    {0}
+};
+
+PyTypeObject PySideSignalMetaType = {
+    PyObject_HEAD_INIT(0)
+    /*ob_size*/             0,
+    /*tp_name*/             "PySide.QtCore.MetaSignal",
+    /*tp_basicsize*/        sizeof(PyTypeObject),
+    /*tp_itemsize*/         0,
+    /*tp_dealloc*/          0,
+    /*tp_print*/            0,
+    /*tp_getattr*/          0,
+    /*tp_setattr*/          0,
+    /*tp_compare*/          0,
+    /*tp_repr*/             0,
+    /*tp_as_number*/        0,
+    /*tp_as_sequence*/      0,
+    /*tp_as_mapping*/       0,
+    /*tp_hash*/             0,
+    /*tp_call*/             0,
+    /*tp_str*/              0,
+    /*tp_getattro*/         0,
+    /*tp_setattro*/         0,
+    /*tp_as_buffer*/        0,
+    /*tp_flags*/            Py_TPFLAGS_DEFAULT,
+    /*tp_doc*/              0,
+    /*tp_traverse*/         0,
+    /*tp_clear*/            0,
+    /*tp_richcompare*/      0,
+    /*tp_weaklistoffset*/   0,
+    /*tp_iter*/             0,
+    /*tp_iternext*/         0,
+    /*tp_methods*/          Signal_methods,
+    /*tp_members*/          0,
+    /*tp_getset*/           0,
+    /*tp_base*/             &PyType_Type,
+};
 
 PyTypeObject PySideSignalType = {
-    PyObject_HEAD_INIT(0)
+    PyObject_HEAD_INIT(&PySideSignalMetaType)
+    //PyObject_HEAD_INIT(0)
     /*ob_size*/             0,
     /*tp_name*/             "PySide.QtCore."SIGNAL_CLASS_NAME,
     /*tp_basicsize*/        sizeof(PySideSignal),
@@ -133,7 +176,7 @@ static PyMappingMethods SignalInstance_as_mapping = {
 PyTypeObject PySideSignalInstanceType = {
     PyObject_HEAD_INIT(0)
     /*ob_size*/             0,
-    /*tp_name*/             "PySide.QtCore."SIGNAL_CLASS_NAME,
+    /*tp_name*/             "PySide.QtCore."SIGNAL_INSTANCE_NAME,
     /*tp_basicsize*/        sizeof(PySideSignalInstance),
     /*tp_itemsize*/         0,
     /*tp_dealloc*/          0,
@@ -152,7 +195,7 @@ PyTypeObject PySideSignalInstanceType = {
     /*tp_setattro*/         0,
     /*tp_as_buffer*/        0,
     /*tp_flags*/            Py_TPFLAGS_DEFAULT,
-    /*tp_doc*/              SIGNAL_CLASS_NAME,
+    /*tp_doc*/              SIGNAL_INSTANCE_NAME,
     /*tp_traverse*/         0,
     /*tp_clear*/            0,
     /*tp_richcompare*/      0,
@@ -285,7 +328,7 @@ PyObject* signalInstanceConnect(PyObject* self, PyObject* args, PyObject* kwds)
     static const char *kwlist[] = {"slot", "type", 0};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds,
-        "O|O:"SIGNAL_CLASS_NAME, (char**) kwlist, &slot, &type))
+        "O|O:"SIGNAL_INSTANCE_NAME, (char**) kwlist, &slot, &type))
         return 0;
 
     PySideSignalInstance *source = reinterpret_cast<PySideSignalInstance*>(self);
@@ -455,12 +498,24 @@ PyObject* signalInstanceCall(PyObject* self, PyObject* args, PyObject* kw)
     return PyCFunction_Call(homonymousMethod, args, kw);
 }
 
+
+static PyObject* metaSignalCheck(PyObject* klass, PyObject* args)
+{
+    if (PyType_IsSubtype(args->ob_type, &PySideSignalInstanceType))
+        Py_RETURN_TRUE;
+    else
+        Py_RETURN_FALSE;
+}
+
 } // extern "C"
 
 namespace PySide { namespace Signal {
 
 void init(PyObject* module)
 {
+    if (PyType_Ready(&PySideSignalMetaType) < 0)
+        return;
+
     if (PyType_Ready(&PySideSignalType) < 0)
         return;
 
@@ -473,6 +528,13 @@ void init(PyObject* module)
     Py_INCREF(&PySideSignalInstanceType);
 }
 
+bool checkType(PyObject* pyObj)
+{
+    if (pyObj)
+        return PyType_IsSubtype(pyObj->ob_type, &PySideSignalType);
+    return false;
+}
+
 void updateSourceObject(PyObject* source)
 {
     PyTypeObject * objType = reinterpret_cast<PyTypeObject *>(PyObject_Type(source));
@@ -509,9 +571,11 @@ char* getTypeName(PyObject* type)
             else if (objType == &PyLong_Type)
                 typeName = strdup("long");
             else if (objType == &PyFloat_Type)
-                typeName = strdup("qreal");
+                typeName = strdup("double");
             else if (objType == &PyBool_Type)
                 typeName = strdup("bool");
+            else if (objType->ob_type == &SbkEnumType_Type)
+                typeName = strdup(Shiboken::Enum::getCppName(objType));
             else
                 typeName = strdup("PyObject");
         }
@@ -526,9 +590,11 @@ char* getTypeName(PyObject* type)
 
 char* buildSignature(const char *name, const char *signature)
 {
-    QString signal;
-    signal.sprintf("%s(%s)", name, signature);
-    return strdup(QMetaObject::normalizedSignature(signal.toAscii()));
+    QByteArray signal(name);
+    signal += '(';
+    signal += signature;
+    signal += ')';
+    return strdup(QMetaObject::normalizedSignature(signal));
 }
 
 char* parseSignature(PyObject *args)
@@ -652,7 +718,11 @@ PySideSignal* newObject(const char* name, ...)
     sig = va_arg(listSignatures, char*);
 
     while(sig != NULL) {
-        appendSignature(self, strdup(sig));
+        if (strcmp(sig, "void") == 0)
+            appendSignature(self, strdup(""));
+        else
+            appendSignature(self, strdup(sig));
+
         sig = va_arg(listSignatures, char*);
     }
 
@@ -661,6 +731,70 @@ PySideSignal* newObject(const char* name, ...)
     return self;
 }
 
+template<typename T>
+static typename T::value_type join(T t, const char* sep)
+{
+    typename T::value_type res;
+    if (!t.size())
+        return res;
+
+    typename T::const_iterator it = t.begin();
+    typename T::const_iterator end = t.end();
+    res += *it;
+    ++it;
+
+    while (it != end) {
+        res += sep;
+        res += *it;
+        ++it;
+    }
+    return res;
+}
+
+static void _addSignalToWrapper(SbkObjectType* wrapperType, const char* signalName, PySideSignal* signal)
+{
+    PyObject* typeDict = wrapperType->super.ht_type.tp_dict;
+    PyObject* homonymousMethod;
+    if ((homonymousMethod = PyDict_GetItemString(typeDict, signalName))) {
+        Py_INCREF(homonymousMethod);
+        signal->homonymousMethod = homonymousMethod;
+    }
+    PyDict_SetItemString(typeDict, signalName, reinterpret_cast<PyObject*>(signal));
+}
+
+void registerSignals(SbkObjectType* pyObj, const QMetaObject* metaObject)
+{
+    typedef QHash<QByteArray, QList<QByteArray> > SignalSigMap;
+    SignalSigMap signalsFound;
+    for(int i = metaObject->methodOffset(), max = metaObject->methodCount(); i < max; ++i) {
+        QMetaMethod method = metaObject->method(i);
+        QByteArray methodName(method.signature());
+        methodName.chop(methodName.size() - methodName.indexOf('('));
+
+        if (method.methodType() == QMetaMethod::Signal)
+            signalsFound[methodName] << join(method.parameterTypes(), ",");
+    }
+
+    SignalSigMap::Iterator it = signalsFound.begin();
+    SignalSigMap::Iterator end = signalsFound.end();
+    for (; it != end; ++it) {
+        PySideSignal* self = PyObject_New(PySideSignal, &PySideSignalType);
+        self->signalName = strdup(it.key().constData());
+        self->signaturesSize = 0;
+        self->signatures = 0;
+        self->initialized = 0;
+        self->homonymousMethod = 0;
+
+        qSort(it.value().begin(), it.value().end());
+        SignalSigMap::mapped_type::const_iterator j = it.value().begin();
+        SignalSigMap::mapped_type::const_iterator endJ = it.value().end();
+        for (; j != endJ; ++j)
+            appendSignature(self, strdup(j->constData()));
+        _addSignalToWrapper(pyObj, it.key(), self);
+        Py_DECREF((PyObject*) self);
+    }
+}
+
 
 PyObject* buildQtCompatible(const char* signature)
 {
@@ -672,13 +806,7 @@ PyObject* buildQtCompatible(const char* signature)
 
 void addSignalToWrapper(SbkObjectType* wrapperType, const char* signalName, PySideSignal* signal)
 {
-    PyObject* typeDict = wrapperType->super.ht_type.tp_dict;
-    PyObject* homonymousMethod;
-    if ((homonymousMethod = PyDict_GetItemString(typeDict, signalName))) {
-        Py_INCREF(homonymousMethod);
-        signal->homonymousMethod = homonymousMethod;
-    }
-    PyDict_SetItemString(typeDict, signalName, reinterpret_cast<PyObject*>(signal));
+    _addSignalToWrapper(wrapperType, signalName, signal);
 }
 
 PyObject* getObject(PySideSignalInstance* signal)
@@ -720,8 +848,8 @@ QStringList getArgsFromSignature(const char* signature, bool* isShortCircuit)
 
 QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* callback, bool encodeName)
 {
-    QString functionName;
-    QString signature;
+    QByteArray functionName;
+    QByteArray signature;
     QStringList args;
     int numArgs = -1;
     bool useSelf = false;
@@ -731,36 +859,37 @@ QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* ca
     if (isMethod || isFunction) {
         PyObject* function = isMethod ? PyMethod_GET_FUNCTION(callback) : callback;
         PyCodeObject* objCode = reinterpret_cast<PyCodeObject*>(PyFunction_GET_CODE(function));
-        functionName = PyString_AS_STRING(objCode->co_name);
+        functionName = PyString_AS_STRING(reinterpret_cast<PyFunctionObject*>(function)->func_name);
         useSelf = isMethod;
         numArgs = objCode->co_flags & CO_VARARGS ? -1 : objCode->co_argcount;
     } else if (PyCFunction_Check(callback)) {
         functionName = ((PyCFunctionObject*)callback)->m_ml->ml_name;
-        useSelf = 0;//((PyCFunctionObject*)callback)->m_self; // commented out to fix bug 736
+        useSelf = ((PyCFunctionObject*)callback)->m_self;
         int flags = ((PyCFunctionObject*)callback)->m_ml->ml_flags;
 
         if (receiver) {
             //Search for signature on metaobject
             const QMetaObject *mo = receiver->metaObject();
+            QByteArray prefix(functionName);
+            prefix += '(';
             for(int i=0; i < mo->methodCount(); i++) {
-            QMetaMethod me = mo->method(i);
-                if (QString(me.signature()).startsWith(functionName)) {
-                    numArgs = me.parameterTypes().size()+1;
+                QMetaMethod me = mo->method(i);
+                if ((strncmp(me.signature(), prefix, prefix.size()) == 0) &&
+                    QMetaObject::checkConnectArgs(signal, me.signature())) {
+                    numArgs = me.parameterTypes().size() + useSelf;
                     break;
                 }
            }
         }
 
         if (numArgs == -1) {
-            if (flags & METH_O)
-                numArgs = 1;
-            else if (flags & METH_VARARGS)
+            if (flags & METH_VARARGS)
                 numArgs = -1;
             else if (flags & METH_NOARGS)
                 numArgs = 0;
         }
     } else if (PyCallable_Check(callback)) {
-        functionName = "__callback"+QString::number((size_t)callback);
+        functionName = "__callback"+QByteArray::number((qlonglong)callback);
     }
 
     Q_ASSERT(!functionName.isEmpty());
@@ -768,7 +897,7 @@ QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* ca
     bool isShortCircuit = false;
 
     if (encodeName)
-        signature = codeCallbackName(callback, functionName);
+        signature = qPrintable(codeCallbackName(callback, functionName));
     else
         signature = functionName;
 
@@ -778,7 +907,7 @@ QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* ca
         signature.append('(');
         if (numArgs == -1)
             numArgs = std::numeric_limits<int>::max();
-        while (args.count() && args.count() > numArgs - useSelf) {
+        while (args.count() && (args.count() > (numArgs - useSelf))) {
             args.removeLast();
         }
         signature.append(args.join(","));
