@@ -25,6 +25,7 @@
 #include "pysideproperty.h"
 #include "pysideproperty_p.h"
 #include "pyside.h"
+#include "dynamicqmetaobject.h"
 
 #include <QHash>
 #include <QStringList>
@@ -42,10 +43,19 @@
 #endif
 #define PYSIDE_SLOT '1'
 #define PYSIDE_SIGNAL '2'
+#include "globalreceiverv2.h"
 #include "globalreceiver.h"
 
 #define PYTHON_TYPE "PyObject"
 
+namespace {
+    static PyObject *metaObjectAttr = 0;
+    static void destroyMetaObject(void* obj)
+    {
+        delete reinterpret_cast<PySide::DynamicQMetaObject*>(obj);
+    }
+}
+
 namespace PySide {
 
 static int callMethod(QObject* object, int id, void** args);
@@ -87,6 +97,57 @@ PyObjectWrapper::operator PyObject*() const
     return m_me;
 }
 
+QDataStream &operator<<(QDataStream& out, const PyObjectWrapper& myObj)
+{
+    if (Py_IsInitialized() == 0) {
+        qWarning() << "Stream operator for PyObject called without python interpreter.";
+        return out;
+    }
+
+    static PyObject *reduce_func  = 0;
+
+    Shiboken::GilState gil;
+    if (!reduce_func) {
+        Shiboken::AutoDecRef pickleModule(PyImport_ImportModule("pickle"));
+        reduce_func = PyObject_GetAttrString(pickleModule, "dumps");
+    }
+    Shiboken::AutoDecRef repr(PyObject_CallFunctionObjArgs(reduce_func, (PyObject*)myObj, NULL));
+    if (repr.object()) {
+        char* buff;
+        Py_ssize_t size;
+        PyString_AsStringAndSize(repr.object(), &buff, &size);
+        QByteArray data(buff, size);
+        out << data;
+    }
+    return out;
+}
+
+QDataStream &operator>>(QDataStream& in, PyObjectWrapper& myObj)
+{
+    if (Py_IsInitialized() == 0) {
+        qWarning() << "Stream operator for PyObject called without python interpreter.";
+        return in;
+    }
+
+    static PyObject *eval_func  = 0;
+
+    Shiboken::GilState gil;
+    if (!eval_func) {
+        Shiboken::AutoDecRef pickleModule(PyImport_ImportModule("pickle"));
+        eval_func = PyObject_GetAttrString(pickleModule, "loads");
+    }
+
+    QByteArray repr;
+    in >> repr;
+    Shiboken::AutoDecRef pyCode(PyString_FromStringAndSize(repr.data(), repr.size()));
+    Shiboken::AutoDecRef value(PyObject_CallFunctionObjArgs(eval_func, pyCode.object(), 0));
+    if (!value.object()) {
+        value = Py_None;
+    }
+    myObj = PyObjectWrapper(value);
+    return in;
+}
+
 };
 
 namespace Shiboken {
@@ -117,7 +178,26 @@ using namespace PySide;
 
 struct SignalManager::SignalManagerPrivate
 {
+    SharedMap m_globalReceivers;
+
+    //Deprecated
     GlobalReceiver m_globalReceiver;
+
+    SignalManagerPrivate()
+    {
+        m_globalReceivers = SharedMap( new QMap<QByteArray, GlobalReceiverV2*>() );
+    }
+
+    ~SignalManagerPrivate()
+    {
+        if (!m_globalReceivers.isNull()) {
+            QList<GlobalReceiverV2*> values = m_globalReceivers->values();
+            m_globalReceivers->clear();
+            if (values.size()) {
+                qDeleteAll(values);
+            }
+        }
+    }
 };
 
 static void clearSignalManager()
@@ -132,11 +212,15 @@ SignalManager::SignalManager() : m_d(new SignalManagerPrivate)
 
     // Register PyObject type to use in queued signal and slot connections
     qRegisterMetaType<PyObjectWrapper>(PYTHON_TYPE);
+    qRegisterMetaTypeStreamOperators<PyObjectWrapper>(PYTHON_TYPE);
 
     TypeResolver::createValueTypeResolver<PyObjectWrapper>(PYTHON_TYPE);
     TypeResolver::createValueTypeResolver<PyObjectWrapper>("object");
     TypeResolver::createValueTypeResolver<PyObjectWrapper>("PySide::PyObjectWrapper");
     PySide::registerCleanupFunction(clearSignalManager);
+
+    if (!metaObjectAttr)
+        metaObjectAttr = PyString_FromString("__METAOBJECT__");
 }
 
 void SignalManager::clear()
@@ -173,7 +257,56 @@ void SignalManager::globalReceiverDisconnectNotify(QObject* source, int slotInde
 
 void SignalManager::addGlobalSlot(const char* slot, PyObject* callback)
 {
-    m_d->m_globalReceiver.addSlot(slot, callback);
+    addGlobalSlotGetIndex(slot, callback);
+}
+
+int SignalManager::addGlobalSlotGetIndex(const char* slot, PyObject* callback)
+{
+    return m_d->m_globalReceiver.addSlot(slot, callback);
+}
+
+QObject* SignalManager::globalReceiver(QObject *sender, PyObject *callback)
+{
+    SharedMap globalReceivers = m_d->m_globalReceivers;
+    QByteArray hash = GlobalReceiverV2::hash(callback);
+    GlobalReceiverV2* gr = 0;
+    if (!globalReceivers->contains(hash) && sender) {
+        gr = (*globalReceivers)[hash] = new GlobalReceiverV2(callback, globalReceivers);
+        gr->incRef(sender); // create a link reference
+        gr->decRef(); // remove extra reference
+    } else {
+        gr = (*globalReceivers)[hash];
+        if (sender)
+            gr->incRef(sender);
+    }
+
+    return reinterpret_cast<QObject*>(gr);
+}
+
+int SignalManager::countConnectionsWith(const QObject *object)
+{
+    int count = 0;
+    foreach(GlobalReceiverV2* g, m_d->m_globalReceivers->values()) {
+        if (g->refCount(object))
+            count++;
+    }
+    return count;
+}
+
+void SignalManager::notifyGlobalReceiver(QObject* receiver)
+{
+    reinterpret_cast<GlobalReceiverV2*>(receiver)->notify();
+}
+
+void SignalManager::releaseGlobalReceiver(const QObject* source, QObject* receiver)
+{
+    GlobalReceiverV2* gr = reinterpret_cast<GlobalReceiverV2*>(receiver);
+    gr->decRef(source);
+}
+
+int SignalManager::globalReceiverSlotIndex(QObject* receiver, const char* signature) const
+{
+    return reinterpret_cast<GlobalReceiverV2*>(receiver)->addSlot(signature);
 }
 
 static bool emitShortCircuitSignal(QObject* source, int signalIndex, PyObject* args)
@@ -254,11 +387,13 @@ int SignalManager::qt_metacall(QObject* object, QMetaObject::Call call, int id,
     PyObject* pp_name = 0;
     QMetaProperty mp;
     PyObject* pySelf = 0;
+    int methodCount = metaObject->methodCount();
+    int propertyCount = metaObject->propertyCount();
 
     if (call != QMetaObject::InvokeMetaMethod) {
         mp = metaObject->property(id);
         if (!mp.isValid())
-            return id - metaObject->methodCount();
+            return id - methodCount;
 
         Shiboken::GilState gil;
         pySelf = (PyObject*)Shiboken::BindingManager::instance().retrieveWrapper(object);
@@ -268,7 +403,7 @@ int SignalManager::qt_metacall(QObject* object, QMetaObject::Call call, int id,
         if (!pp) {
             qWarning("Invalid property: %s.", mp.name());
             Py_XDECREF(pp_name);
-            return id - metaObject->methodCount();
+            return id - methodCount;
         }
     }
 
@@ -293,10 +428,13 @@ int SignalManager::qt_metacall(QObject* object, QMetaObject::Call call, int id,
             qWarning("Unsupported meta invocation type.");
     }
 
+    // WARNING Isn't safe to call any metaObject and/or object methods beyond this point
+    //         because the object can be deleted inside the called slot.
+
     if (call == QMetaObject::InvokeMetaMethod)
-        id = id - metaObject->methodCount();
+        id = id - methodCount;
     else
-        id = id - metaObject->propertyCount();
+        id = id - propertyCount;
 
     if (pp || pp_name) {
         Shiboken::GilState gil;
@@ -360,8 +498,13 @@ static int PySide::callMethod(QObject* object, int id, void** args)
     }
     return -1;
 }
-
 bool SignalManager::registerMetaMethod(QObject* source, const char* signature, QMetaMethod::MethodType type)
+{
+    int ret = registerMetaMethodGetIndex(source, signature, type);
+    return (ret != -1);
+}
+
+int SignalManager::registerMetaMethodGetIndex(QObject* source, const char* signature, QMetaMethod::MethodType type)
 {
     Q_ASSERT(source);
     const QMetaObject* metaObject = source->metaObject();
@@ -371,19 +514,53 @@ bool SignalManager::registerMetaMethod(QObject* source, const char* signature, Q
         SbkObject* self = Shiboken::BindingManager::instance().retrieveWrapper(source);
         if (!Shiboken::Object::hasCppWrapper(self)) {
             qWarning() << "Invalid Signal signature:" << signature;
-            return false;
+            return -1;
         } else {
-            PySide::DynamicQMetaObject* dynMetaObj = reinterpret_cast<PySide::DynamicQMetaObject*>(const_cast<QMetaObject*>(metaObject));
+            DynamicQMetaObject *dmo = 0;
+            PyObject *pySelf = reinterpret_cast<PyObject*>(self);
+            PyObject* dict = self->ob_dict;
+
+            // Create a instance meta object
+            if (!dict || !PyDict_Contains(dict, metaObjectAttr)) {
+                dmo = new DynamicQMetaObject(pySelf->ob_type, metaObject);
+                PyObject *pyDmo = PyCObject_FromVoidPtr(dmo, destroyMetaObject);
+                PyObject_SetAttr(pySelf, metaObjectAttr, pyDmo);
+                Py_DECREF(pyDmo);
+            } else {
+                dmo = reinterpret_cast<DynamicQMetaObject*>(const_cast<QMetaObject*>(metaObject));
+            }
+
             if (type == QMetaMethod::Signal)
-                dynMetaObj->addSignal(signature);
+                return dmo->addSignal(signature);
             else
-                dynMetaObj->addSlot(signature);
+                return dmo->addSlot(signature);
         }
     }
-    return true;
+    return methodIndex;
 }
 
 bool SignalManager::hasConnectionWith(const QObject *object)
 {
     return m_d->m_globalReceiver.hasConnectionWith(object);
 }
+
+
+const QMetaObject* SignalManager::retriveMetaObject(PyObject *self)
+{
+    Shiboken::GilState gil;
+    DynamicQMetaObject *mo = 0;
+    Q_ASSERT(self);
+
+    PyObject* dict = reinterpret_cast<SbkObject*>(self)->ob_dict;
+    if (dict && PyDict_Contains(dict, metaObjectAttr)) {
+        PyObject *pyMo = PyDict_GetItem(dict, metaObjectAttr);
+        mo = reinterpret_cast<DynamicQMetaObject*>(PyCObject_AsVoidPtr(pyMo));
+    } else {
+        mo = reinterpret_cast<DynamicQMetaObject*>(Shiboken::Object::getTypeUserData(reinterpret_cast<SbkObject*>(self)));
+    }
+
+    mo->update();
+    return mo;
+}
+
+
